#pragma once

#include <FlashTool/FlashTool.h>
#include "FlvDataCollector.h"
#include "MediaDetector.h"
#include "Grabber.h"

class ConvertorObserver;

//=====================================================================================//
//                                   class Convertor                                   //
//=====================================================================================//
class Convertor : public FlvDataCollectorObserver
{
private:

	typedef ConvertorObserver Observer;

	MediaDetector m_mediaDetector;
	Observer* m_observer;
	Grabber m_grabber;

public:
	Convertor(const std::string& file_name);
	~Convertor();
public:
	///	convert open avi file to flv
	bool convertToFlv(const std::string& flv_file_name);
	///	set observer
	void setObserver(ConvertorObserver* observer);
	///	get media detector
	const MediaDetector& getMediaDetector() const;

private:
	///	setup FlashTool structure
	void setupFlashToolStreamInfo(FTFLVSTREAMINFO& ftflvsi,FTVIDEOINFO& ftvi,FTAUDIOINFO& ftai);
	///	process progress of converting
	void onProgress(unsigned int timestamp);
	///	setup grabber options
	unsigned int setupGrabberOptions();

};

//	get media detector
inline const MediaDetector& Convertor::getMediaDetector() const
{
	return m_mediaDetector;
}

//=====================================================================================//
//                               class ConvertorObserver                               //
//=====================================================================================//
class ConvertorObserver
{
public:
	virtual ~ConvertorObserver() {}
	///	process progress of converting
	virtual void onProgress(unsigned int timestamp) = 0;

};

