#include "StdAfx.h"
#include "MediaDetector.h"

#include "Utility.h"

//=====================================================================================//
//                                 class MediaDetector                                 //
//=====================================================================================//
MediaDetector::MediaDetector(const std::string& file_name)
: m_duration(0)
{
	HRESULT hr = E_FAIL;

	ATL::CComPtr<IMediaDet> mDetector;


	if(FAILED(hr = mDetector.CoCreateInstance(CLSID_MediaDet)))
		throw std::exception("can't create instance of media detector interface");

	if(FAILED(hr = mDetector->put_Filename(CComBSTR(file_name.c_str()))))
		throw std::exception(("can't found file [" + file_name + "]").c_str());

	long streams = 0;
	mDetector->get_OutputStreams(&streams);

	for(long i=0;i<streams;++i)
	{
		mDetector->put_CurrentStream(i);

		AM_MEDIA_TYPE amt;
		if(!FAILED(mDetector->get_StreamMediaType(&amt)))
		{
			if(amt.majortype == MEDIATYPE_Video)
			{
				if((amt.formattype == FORMAT_VideoInfo) && 
				   (amt.cbFormat >= sizeof(VIDEOINFOHEADER)) &&
				   (amt.pbFormat != 0))
				{
					double frame_rate = 0;
					mDetector->get_FrameRate(&frame_rate);
					m_videoDataFormat.reset(new MDVideoDataFormat(reinterpret_cast<VIDEOINFOHEADER*>(amt.pbFormat),
																  static_cast<unsigned int>(frame_rate)));
				}
			}

			if(amt.majortype == MEDIATYPE_Audio)
			{
				if((amt.formattype == FORMAT_WaveFormatEx) && 
				   (amt.cbFormat >= sizeof(WAVEFORMATEX)) &&
				   (amt.pbFormat != 0) )
				{
					m_audioDataFormat.reset(new MDAudioDataFormat(reinterpret_cast<WAVEFORMATEX*>(amt.pbFormat)));
				}
			}

			double length;
			mDetector->get_StreamLength(&length);
			m_duration = std::max<unsigned int>(m_duration,static_cast<unsigned int>(length*1000));

			Utility::FreeMediaType(amt);
		}
	}
	
}

MediaDetector::~MediaDetector()
{
}

//=====================================================================================//
//                               class MDVideoDataFormat                               //
//=====================================================================================//
MDVideoDataFormat::MDVideoDataFormat(const VIDEOINFOHEADER* vih,unsigned int frame_rate)
: m_frameRate(frame_rate)
, m_width(vih->bmiHeader.biWidth)
, m_height(vih->bmiHeader.biHeight)
, m_bitCount(vih->bmiHeader.biBitCount)
{
}

MDVideoDataFormat::~MDVideoDataFormat()
{
}

//=====================================================================================//
//                               class MDAudioDataFormat                               //
//=====================================================================================//
MDAudioDataFormat::MDAudioDataFormat(const WAVEFORMATEX* vih)
: m_samplesPerSec(vih->nSamplesPerSec)
, m_bitsPerSample(vih->wBitsPerSample)
, m_formatTag(vih->wFormatTag)
, m_channels(vih->nChannels)
{
}

MDAudioDataFormat::~MDAudioDataFormat()
{
}
