Imports System.IO
Imports System.Runtime.InteropServices

Public Class FlvCompiler
    Implements IDisposable

    Private m_width As Integer = 0
    Private m_height As Integer = 0
    Private m_file As FileStream = Nothing
    Private m_hFlvStream As IntPtr = IntPtr.Zero
    Private m_hScreenVideoEncoder As IntPtr = IntPtr.Zero
    Private m_ftWriteDataCallback As NativeMethods.FTWRITEDATACALLBACK = Nothing

    ' Keep track of when the object is disposed.
    Protected m_disposed As Boolean = False

    Public Sub New(ByVal fileName As String)
        m_file = New FileStream(fileName, FileMode.Create)
        m_ftWriteDataCallback = AddressOf WriteToStreamCallback
    End Sub
    ' Implements IDisposable.Dispose.
    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
    ' Add frame to flv stream
    Public Function AddFrame(ByVal imageFileName As String, ByVal timestamp As Integer) As Boolean
        Try
            Dim bitmap As New Bitmap(imageFileName)

            If bitmap.PixelFormat = Drawing.Imaging.PixelFormat.Format24bppRgb Then
                If CheckFlvStream(bitmap.Width, bitmap.Height) Then
                    If m_hScreenVideoEncoder <> IntPtr.Zero Then
                        ' lock the bitmap's bits.  
                        Dim bmRect As New Rectangle(0, 0, bitmap.Width, bitmap.Height)
                        Dim bmData As Drawing.Imaging.BitmapData = bitmap.LockBits(bmRect, Drawing.Imaging.ImageLockMode.ReadOnly, bitmap.PixelFormat)

                        ' get the address of the first line.
                        Dim bmDataPtr As IntPtr = bmData.Scan0
                        Dim bmDataSize As Int32 = bmData.Height * bmData.Stride

                        ' encode the video frame
                        NativeMethods.ftScreenVideoEncodeFrame(m_hScreenVideoEncoder, bmDataPtr, bmDataSize, False)

                        ' unlock the bitmap's bits
                        bitmap.UnlockBits(bmData)

                        ' get the encoded video frame
                        Dim lpData As IntPtr = IntPtr.Zero
                        Dim cbBytes As Int32 = 0
                        NativeMethods.ftScreenVideoGetEncodedFrame(m_hScreenVideoEncoder, lpData, cbBytes)

                        ' add frame to Flv stream
                        NativeMethods.ftFlvStreamPutVideoData(m_hFlvStream, lpData, cbBytes, timestamp)
                        AddFrame = True
                        Exit Function
                    End If
                End If
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "FlvComposer", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

        AddFrame = False
    End Function
    ' This method disposes the base object's resources.
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.m_disposed Then
            If disposing Then
                ' release Screen Video encoder
                If m_hScreenVideoEncoder <> IntPtr.Zero Then
                    NativeMethods.ftScreenVideoEncoderDestroy(m_hScreenVideoEncoder)
                    m_hScreenVideoEncoder = IntPtr.Zero
                End If
                ' release Flv Stream
                If m_hFlvStream <> IntPtr.Zero Then
                    NativeMethods.ftFlvStreamDestroy(m_hFlvStream)
                    m_hFlvStream = IntPtr.Zero
                End If
                ' close output file
                m_file.Close()
            End If
        End If
        Me.m_disposed = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub
    '	check flv stream state and parameters
    Private Function CheckFlvStream(ByVal width As Integer, ByVal height As Integer) As Boolean
        If m_hFlvStream = IntPtr.Zero Then
            '	initialize FTVIDEOINFO struct
            Dim ftvi As New NativeMethods.FTVIDEOINFO()
            ftvi.dwCodec = NativeMethods.FT_VIDEOCODEC_SCREENVIDEO
            ftvi.dwWidth = width
            ftvi.dwHeight = height

            '	initialize FTFLVSTREAMINFO struct
            Dim gchFtvi As GCHandle = GCHandle.Alloc(ftvi, GCHandleType.Pinned)
            Dim ftflvsi As New NativeMethods.FTFLVSTREAMINFO()
            ftflvsi.lpVideoInfo = gchFtvi.AddrOfPinnedObject()
            ftflvsi.lpAudioInfo = IntPtr.Zero
            ftflvsi.lpWriteDataCallback = m_ftWriteDataCallback
            ftflvsi.lpParam = IntPtr.Zero

            '	create flv stream
            m_hFlvStream = NativeMethods.ftFlvStreamCreate(ftflvsi)
            gchFtvi.Free()

            '	create Screen Video encoder
            Const block_size As Integer = 16
            m_hScreenVideoEncoder = NativeMethods.ftScreenVideoEncoderCreate(width, height, block_size)

            m_width = width
            m_height = height
        End If

        CheckFlvStream = (m_hFlvStream <> IntPtr.Zero) And (m_width = width) And (m_height = height)
    End Function

    Private Sub WriteToStreamCallback(ByVal lpParam As IntPtr, ByVal lpData As IntPtr, ByVal nLen As Int32)
        If nLen <> 0 Then
            Dim buffer(nLen) As Byte
            Marshal.Copy(lpData, buffer, 0, nLen)
            m_file.Write(buffer, 0, nLen)
        End If
    End Sub
End Class
